-- Standard awesome library
require("awful")
require("awful.autofocus")
require("awful.rules")
-- Theme handling library
require("beautiful")
-- Notification library
require("naughty")

require("vicious")

-- Load Debian menu entries
require("debian.menu")

-- {{{ Variable definitions
-- Themes define colours, icons, and wallpapers
-- The default is a dark theme
theme_path = "/usr/share/awesome/themes/default/theme.lua"
-- Uncommment this for a lighter theme
-- theme_path = "/usr/share/awesome/themes/sky/theme.lua"

wallpaper_cmd = { "awsetbg -t .config/awesome/bgstripes.png" } 

-- Actually load theme
beautiful.init(theme_path)

-- This is used later as the default terminal and editor to run.
terminal = "x-terminal-emulator"
editor = os.getenv("EDITOR") or "editor"
editor_cmd = terminal .. " -e " .. editor

-- Default modkey.
-- Usually, Mod4 is the key with a logo between Control and Alt.
-- If you do not like this or do not have such a key,
-- I suggest you to remap Mod4 to another key using xmodmap or other tools.
-- However, you can use another modifier like Mod1, but it may interact with others.
modkey = "Mod4"

-- Table of layouts to cover with awful.layout.inc, order matters.
layouts =
{
--    awful.layout.suit.tile,
--    awful.layout.suit.tile.left,
--    awful.layout.suit.tile.bottom,
--    awful.layout.suit.tile.top,
    awful.layout.suit.fair,
    awful.layout.suit.fair.horizontal,
--    awful.layout.suit.spiral,
--    awful.layout.suit.spiral.dwindle,
    awful.layout.suit.max,
--    awful.layout.suit.max.fullscreen,
    awful.layout.suit.magnifier,
    awful.layout.suit.floating
}

---- Table of clients that should be set floating. The index may be either
---- the application class or instance. The instance is useful when running
---- a console app in a terminal like (Music on Console)
----    xterm -name mocp -e mocp
---- OVERRULED BY TILEDAPPS BELOW
--floatapps =
--{
--    -- by class
--    ["MPlayer"] = true,
--    ["pinentry"] = true,
--    ["GIMP"] = true,
--    ["twinkle"] = true,
--    ["Add-ons"] = true,
--    ["Play stream"] = true,
--    ["gscan2pdf"] = true,
--}
--
---- Applications that should never float, assuming everything else floats
---- (by instance)
--tiledapps =
--{
--    ["urxvt"] = true,
--}
--
---- Applications that should be maximised
---- (by instance)
--maxapps =
--{
--    ["Navigator"] = true,
--    -- jpilot is -v
--    ["-v"] = true,
--    ["Xpdf"] = true,
--    ["gscan2pdf"] = true
--}
--
---- Applications to be moved to a pre-defined tag by class or instance.
---- Use the screen and tags indices.
--apptags =
--{
--    ["Navigator"] = { screen = 1, tag = 9 },
--    -- jpilot is -v
--    ["-v"] = { screen = 1, tag = 8 },
--}

-- Define if we want to use titlebar on all applications.
use_titlebar = false
-- }}}

-- {{{ Tags
tags = {}
tags.settings = {
    { name = "1", layout = layouts[1] },
    { name = "2", layout = layouts[1] },
    { name = "3", layout = layouts[1] },
    { name = "4", layout = layouts[1] },
    { name = "5", layout = layouts[5] },
    { name = "6", layout = layouts[3] },
    { name = "7", layout = layouts[3] },
    { name = "8", layout = layouts[3] },
    { name = "9", layout = layouts[3] },
}

-- Define a tag table which hold all screen tags.
for s = 1, screen.count() do
    tags[s] = {}
    for i, v in ipairs(tags.settings) do
        tags[s][i] = tag({ name = v.name })
        tags[s][i].screen = s
        awful.tag.setproperty(tags[s][i], "layout", v.layout)
        awful.tag.setproperty(tags[s][i], "mwfact", v.mwfact)
        awful.tag.setproperty(tags[s][i], "hide",   v.hide)
    end
    tags[s][1].selected = true
end

if screen.count() == 3 then
  tags[1][1].selected = false
  tags[1][9].selected = true
end
-- }}}

-- {{{ Menu
-- Create a laucher widget and a main menu
myawesomemenu = {
   { "manual", terminal .. " -e man awesome" },
   { "edit config", editor_cmd .. " " .. awful.util.getdir("config") .. "/rc.lua" },
   { "restart", awesome.restart },
   { "quit", awesome.quit }
}

mymainmenu = awful.menu({ items = { { "awesome", myawesomemenu, beautiful.awesome_icon },
                                    { "Debian", debian.menu.Debian_menu.Debian },
                                    { "open terminal", terminal }
                                  }
                        })

mylauncher = awful.widget.launcher({ image = image(beautiful.awesome_icon),
                                     menu = mymainmenu })
-- }}}

-- {{{ Wibox

-- {{{ Reusable separators
spacer         = widget({ type = "textbox", name = "spacer" })
separator      = widget({ type = "textbox", name = "separator" })
spacer.text    = " "
separator.text = "٭"
-- }}}

---- {{{ CPU usage and temperature
---- Widget icon
--cpuicon        = widget({ type = "imagebox", name = "cpuicon" })
--cpuicon.image  = image(beautiful.widget_cpu)
---- Initialize widgets
--thermalwidget  = widget({ type = "textbox", name = "thermalwidget" })
--cpuwidget      = awful.widget.graph({ layout = awful.widget.layout.horizontal.rightleft })
---- Graph properties
--cpuwidget:set_width(50)
----cpuwidget:set_scale(false)
--cpuwidget:set_max_value(100)
--cpuwidget:set_background_color(beautiful.fg_off_widget)
--cpuwidget:set_border_color(beautiful.border_widget)
--cpuwidget:set_color(beautiful.fg_end_widget)
--cpuwidget:set_gradient_angle(0)
--cpuwidget:set_gradient_colors({
--    beautiful.fg_end_widget,
--    beautiful.fg_center_widget,
--    beautiful.fg_widget })
---- Register widgets
--vicious.register(cpuwidget, vicious.widgets.cpu, "$1")
--vicious.register(thermalwidget, vicious.widgets.thermal, "CPU: $1°C", 19, "hwmon0")
---- }}}

-- Battery widget
--batterywidget = widget({ type = 'progressbar', name = 'batterywidget' })
--batterywidget.width = 100
--batterywidget.height = 0.8
--batterywidget.gap = 1
--batterywidget.border_padding = 1
--batterywidget.border_width = 1
--batterywidget.ticks_count = 10
--batterywidget.ticks_gap = 1
--batterywidget.vertical = false
--batterywidget:bar_properties_set('bat', {
--  bg = 'black',
--  fg = 'blue4',
--  fg_off = 'red',
--  reverse = false,
--  min_value = 0,
--  max_value = 100
--})

-- Create a textclock widget
mytextclock = awful.widget.textclock({ align = "right" }, "%a %d %b %H:%M:%S", 1)

-- Create a systray
mysystray = widget({ type = "systray", align = "right" })

mybatterybox = widget({ type = "textbox", name = "mybatterybox", align = "right" })

-- Create a wibox for each screen and add it
mywibox = {}
mypromptbox = {}
mylayoutbox = {}
mytaglist = {}
mytaglist.buttons = awful.util.table.join(
                    awful.button({ }, 1, awful.tag.viewonly),
                    awful.button({ modkey }, 1, awful.client.movetotag),
                    awful.button({ }, 3, awful.tag.viewtoggle),
                    awful.button({ modkey }, 3, awful.client.toggletag),
                    awful.button({ }, 4, awful.tag.viewnext),
                    awful.button({ }, 5, awful.tag.viewprev)
                    )
mytasklist = {}
mytasklist.buttons = awful.util.table.join(
                     awful.button({ }, 1, function (c)
                                              if not c:isvisible() then
                                                  awful.tag.viewonly(c:tags()[1])
                                              end
                                              client.focus = c
                                              c:raise()
                                          end),
                     awful.button({ }, 3, function ()
                                              if instance then
                                                  instance:hide()
                                                  instance = nil
                                              else
                                                  instance = awful.menu.clients({ width=250 })
                                              end
                                          end),
                     awful.button({ }, 4, function ()
                                              awful.client.focus.byidx(1)
                                              if client.focus then client.focus:raise() end
                                          end),
                     awful.button({ }, 5, function ()
                                              awful.client.focus.byidx(-1)
                                              if client.focus then client.focus:raise() end
                                          end))

for s = 1, screen.count() do
    -- Create a promptbox for each screen
    mypromptbox[s] = awful.widget.prompt({ layout = awful.widget.layout.horizontal.leftright })
    -- Create an imagebox widget which will contains an icon indicating which layout we're using.
    -- We need one layoutbox per screen.
    mylayoutbox[s] = awful.widget.layoutbox(s)
    mylayoutbox[s]:buttons(awful.util.table.join(
                           awful.button({ }, 1, function () awful.layout.inc(layouts, 1) end),
                           awful.button({ }, 3, function () awful.layout.inc(layouts, -1) end),
                           awful.button({ }, 4, function () awful.layout.inc(layouts, 1) end),
                           awful.button({ }, 5, function () awful.layout.inc(layouts, -1) end)))
    -- Create a taglist widget
    mytaglist[s] = awful.widget.taglist(s, awful.widget.taglist.label.all, mytaglist.buttons)

    -- Create a tasklist widget
    mytasklist[s] = awful.widget.tasklist(function(c)
                                              return awful.widget.tasklist.label.currenttags(c, s)
                                          end, mytasklist.buttons)

    -- Create the wibox
    mywibox[s] = awful.wibox({ position = "top", screen = s })
    -- Add widgets to the wibox - order matters
    mywibox[s].widgets = {
        {
--            mylauncher,
            mytaglist[s],
            mypromptbox[s],
            layout = awful.widget.layout.horizontal.leftright
        },
        mylayoutbox[s],
        spacer,
        mytextclock,
        spacer, separator, spacer,
        mybatterybox,
--        batterywidget,
--        spacer, separator, spacer,
--        cpuwidget.widget,
--        spacer, separator, spacer,
--        thermalwidget.widget,
        spacer,
        s == screen.count() and mysystray or nil,
        mytasklist[s],
        layout = awful.widget.layout.horizontal.rightleft
    }
end

-- }}}

-- {{{ Mouse bindings
root.buttons(awful.util.table.join(
    awful.button({ }, 3, function () mymainmenu:toggle() end),
    awful.button({ }, 4, awful.tag.viewnext),
    awful.button({ }, 5, awful.tag.viewprev)
))
-- }}}

-- {{{ Key bindings
globalkeys = awful.util.table.join(
    awful.key({ modkey,           }, "Left",   awful.tag.viewprev       ),
    awful.key({ modkey,           }, "Right",  awful.tag.viewnext       ),
    awful.key({ modkey,           }, "Escape", awful.tag.history.restore),
    awful.key({ modkey, "Shift"   }, "Right", function () awful.screen.focus( 1)       end),
    awful.key({ modkey, "Shift"   }, "Left", function () awful.screen.focus(-1)       end),
    awful.key({ modkey, "Shift"   }, "h",   awful.tag.viewprev       ),
    awful.key({ modkey, "Shift"   }, "l",  awful.tag.viewnext       ),

    awful.key({ modkey,           }, "k",
        function ()
            awful.client.focus.byidx( 1)
            if client.focus then client.focus:raise() end
        end),
    awful.key({ modkey,           }, "j",
        function ()
            awful.client.focus.byidx(-1)
            if client.focus then client.focus:raise() end
        end),
    awful.key({ modkey,           }, "w", function () mymainmenu:show(true)        end),

    -- Layout manipulation
    awful.key({ modkey, "Shift"   }, "k", function () awful.client.swap.byidx(  1) end),
    awful.key({ modkey, "Shift"   }, "j", function () awful.client.swap.byidx( -1) end),
    awful.key({ modkey, "Control" }, "k", function () awful.screen.focus( 1)       end),
    awful.key({ modkey, "Control" }, "j", function () awful.screen.focus(-1)       end),
    awful.key({ modkey,           }, "u", awful.client.urgent.jumpto),
    awful.key({ modkey,           }, "Tab",
        function ()
            awful.client.focus.history.previous()
            if client.focus then
                client.focus:raise()
            end
        end),

    -- Standard program
    awful.key({ modkey,           }, "Return", function () awful.util.spawn(terminal) end),
    awful.key({ modkey, "Control" }, "r", awesome.restart),
    awful.key({ modkey, "Shift"   }, "q", awesome.quit),

    awful.key({ modkey,           }, "l",     function () awful.tag.incmwfact( 0.05)    end),
    awful.key({ modkey,           }, "h",     function () awful.tag.incmwfact(-0.05)    end),
    awful.key({ modkey, "Shift"   }, "h",     function () awful.tag.incnmaster( 1)      end),
    awful.key({ modkey, "Shift"   }, "l",     function () awful.tag.incnmaster(-1)      end),
    awful.key({ modkey, "Control" }, "h",     function () awful.tag.incncol( 1)         end),
    awful.key({ modkey, "Control" }, "l",     function () awful.tag.incncol(-1)         end),
    awful.key({ modkey,           }, "space", function () awful.layout.inc(layouts,  1) end),
    awful.key({ modkey, "Shift"   }, "space", function () awful.layout.inc(layouts, -1) end),

    -- Prompt
    awful.key({ modkey },            "r",     function () mypromptbox[mouse.screen]:run() end),

    awful.key({ modkey }, "x",
              function ()
                  awful.prompt.run({ prompt = "Run Lua code: " },
                  mypromptbox[mouse.screen].widget,
                  awful.util.eval, nil,
                  awful.util.getdir("cache") .. "/history_eval")
              end)
)

clientkeys = awful.util.table.join(
    awful.key({ modkey,           }, "f",      function (c) c.fullscreen = not c.fullscreen  end),
    awful.key({ modkey, "Shift"   }, "c",      function (c) c:kill()                         end),
    awful.key({ modkey, "Control" }, "space",  awful.client.floating.toggle                     ),
    awful.key({ modkey, "Control" }, "Return", function (c) c:swap(awful.client.getmaster()) end),
    awful.key({ modkey,           }, "o",      awful.client.movetoscreen                        ),
    awful.key({ modkey, "Shift"   }, "r",      function (c) c:redraw()                       end),
    awful.key({ modkey            }, "t",      awful.client.togglemarked                        ),
    awful.key({ modkey,           }, "n",      function (c) c.minimized = not c.minimized    end),
    awful.key({ modkey,           }, "m",
        function (c)
            c.maximized_horizontal = not c.maximized_horizontal
            c.maximized_vertical   = not c.maximized_vertical
        end)
)

-- Compute the maximum number of digit we need, limited to 9
keynumber = 0
for s = 1, screen.count() do
   keynumber = math.min(9, math.max(#tags[s], keynumber));
end

-- Bind all key numbers to tags.
-- Be careful: we use keycodes to make it works on any keyboard layout.
-- This should map on the top row of your keyboard, usually 1 to 9.
for i = 1, keynumber do
    globalkeys = awful.util.table.join(globalkeys,
        awful.key({ modkey }, "#" .. i + 9,
                  function ()
                        local screen = mouse.screen
                        if tags[screen][i] then
                            awful.tag.viewonly(tags[screen][i])
                        end
                  end),
        awful.key({ modkey, "Control" }, "#" .. i + 9,
                  function ()
                      local screen = mouse.screen
                      if tags[screen][i] then
                          awful.tag.viewtoggle(tags[screen][i])
                      end
                  end),
        awful.key({ modkey, "Shift" }, "#" .. i + 9,
                  function ()
                      if client.focus and tags[client.focus.screen][i] then
                          awful.client.movetotag(tags[client.focus.screen][i])
                      end
                  end),
        awful.key({ modkey, "Control", "Shift" }, "#" .. i + 9,
                  function ()
                      if client.focus and tags[client.focus.screen][i] then
                          awful.client.toggletag(tags[client.focus.screen][i])
                      end
                  end))
end

clientbuttons = awful.util.table.join(
    awful.button({ }, 1, function (c) client.focus = c; c:raise() end),
    awful.button({ modkey }, 1, awful.mouse.client.move),
    awful.button({ modkey }, 3, awful.mouse.client.resize))

cmdmodkey = "Mod3"

-- xmms2 & sound
globalkeys = awful.util.table.join(globalkeys,
  awful.key({ cmdmodkey }, "Prior", function () awful.util.spawn("amixer set Master 2+") end),
  awful.key({ cmdmodkey }, "Next", function () awful.util.spawn("amixer set Master 2-") end),
  awful.key({ cmdmodkey }, "Up", function () awful.util.spawn("amixer set PCM 2+") end),
  awful.key({ cmdmodkey }, "Down", function () awful.util.spawn("amixer set PCM 2-") end),
  awful.key({ cmdmodkey }, "Home", function () awful.util.spawn("amixer set Mic toggle") end),
  awful.key({ cmdmodkey }, "End", function () awful.util.spawn("amixer set Master toggle") end),
  awful.key({ cmdmodkey }, "Left", function () awful.util.spawn("xmms2 prev") end),
  awful.key({ cmdmodkey }, "Right", function () awful.util.spawn("xmms2 next") end),
  awful.key({ cmdmodkey }, "space", function () awful.util.spawn("xmms2 toggleplay") end),
  awful.key({ cmdmodkey }, "backslash", function ()
    local f = io.popen('xmms2 current', 'r')
    naughty.notify({ title = "Now playing", text = f:read(), timeout = 5 })
    f:close()
  end),
  awful.key({ cmdmodkey, "Shift" }, "backslash", function ()
    local f = io.popen('xmms2 list', 'r')
    naughty.notify({ title = "Playlist", text = f:read(), timeout = 15 })
    f:close()
  end)
)

-- misc apps
globalkeys = awful.util.table.join(globalkeys,
  awful.key({ cmdmodkey }, "n", function () awful.util.spawn("sensible-browser") end),
  awful.key({ cmdmodkey }, "m", function () awful.util.spawn(terminal .. " -e mutt -f =store") end),
  awful.key({ cmdmodkey }, "t", function () awful.util.spawn(terminal) end),
  awful.key({ cmdmodkey }, "c", function () awful.util.spawn(terminal .. " -e python") end),
  awful.key({ cmdmodkey }, "r", function () mypromptbox[mouse.screen]:run() end),
  awful.key({ cmdmodkey }, "j", function () awful.util.spawn("jpilot") end),
  awful.key({ cmdmodkey }, "i", function () awful.util.spawn(terminal .. " -title irc -name irc -e ssh -Snone -t irc screen -dr irc") end),
  awful.key({ cmdmodkey }, "x", function () awful.util.spawn_with_shell("/sbin/start-stop-daemon --start --background --exec /usr/bin/xscreensaver; xscreensaver-command -lock") end),
  awful.key({ cmdmodkey, "Shift" }, "x", function () awful.util.spawn("xscreensaver-command -exit") end),
  awful.key(nil, "XF86ScreenSaver", function () awful.util.spawn("xset dpms force off") end)
)

-- Set keys
root.keys(globalkeys)
-- }}}

-- {{{ Rules
awful.rules.rules = {
    -- All clients will match this rule.
    { rule = { },
      properties = { border_width = beautiful.border_width,
                     border_color = beautiful.border_normal,
                     focus = true,
                     floating = true,
                     keys = clientkeys,
                     buttons = clientbuttons } },
    { rule = { class = "URxvt" },
      properties = { floating = false } },
    { rule = { class = "URxvt", instance = "irc" },
      properties = { floating = false, tag = tags[screen.count()][screen.count() == 1 and 2 or 1], switchtotag = true } },
    { rule = { class = "Iceweasel" },
      properties = { tag = tags[screen.count() == 3 and 1 or screen.count()][9], switchtotag = false, floating = false } },
    { rule = { class = "Iceweasel", instance = "Dialog" },
      properties = { floating = true }, callback = awful.placement.centered },
    { rule = { class = "Jpilot" },
      properties = { tag = tags[screen.count() == 3 and 1 or screen.count()][8], switchtotag = true, floating = false } },
}
-- }}}

-- {{{ Signals
-- Signal function to execute when a new client appears.
client.add_signal("manage", function (c, startup)
    -- Add a titlebar
    -- awful.titlebar.add(c, { modkey = modkey })

    -- Enable sloppy focus
    c:add_signal("mouse::enter", function(c)
        if awful.layout.get(c.screen) ~= awful.layout.suit.magnifier
            and awful.client.focus.filter(c) then
            client.focus = c
        end
    end)

    if not startup then
        -- Set the windows at the slave,
        -- i.e. put it at the end of others instead of setting it master.
        -- awful.client.setslave(c)

        -- Put windows in a smart way, only if they does not set an initial position.
        if not c.size_hints.user_position and not c.size_hints.program_position then
            --awful.placement.center_vertical(c)
            --awful.placement.center_horizontal(c)
            awful.placement.no_overlap(c)
            awful.placement.no_offscreen(c)
        end
    end
end)

client.add_signal("focus", function(c) c.border_color = beautiful.border_focus end)
client.add_signal("unfocus", function(c) c.border_color = beautiful.border_normal end)

-- Hook called every sixty seconds
function hook_battery()
    mybatterybox.text = " " .. get_acpibatt() .. " "
end
-- }}}

-- {{{ Statusbar battery
--
function get_acpibatt()
    local f = io.popen('acpi -b', 'r')
    if not f then
      return "acpi -b failed"
    end

    local s = f:read('*l')
    f:close()
    if not s then
      return '-';
    end

    -- Battery 0: Discharging, 89%, 00:02:14 remaining
    -- Battery 0: Charging, 58%, 00:02:14 until charged
    -- Battery 0: Full, 100%
    -- so find the first bit first and then go look for the time
    local st, en, status, percent = string.find(s, '%a+%s%d:%s(%a+),%s(%d+%%)');
    local st, en, time = string.find(s, ',%s(%d+:%d+):%d+%s%a+', en);

    if not status or not percent then -- time can be empty if we're full
      return "couldn't parse line " .. s
    end

    if not time then
      return percent
    end

    if status == 'Charging' then
      return '↑ ' .. percent;
    elseif status == 'Discharging' then
      return '↓ '.. time;
    else
      return '';
    end
end
t = timer({ timeout = 20 })
t:add_signal('timeout', hook_battery)
t:start()
hook_battery()
-- }}}

--{{{ batt hook
local function get_bat()
  local a = io.open("/sys/class/power_supply/BAT1/charge_full")
  for line in a:lines() do
    full = line
  end
  a:close()
  local b = io.open("/sys/class/power_supply/BAT1/charge_now")
  for line in b:lines() do
    now = line
  end
  b:close()
  batt=math.floor(now*100/full)
  batterywidget:bar_data_add("bat",batt )
end
-- awful.hooks.timer.register(5, get_bat)
--}}}
